<?php
namespace sv_media_library;

/**
 * @version         1.00
 * @author			straightvisions GmbH
 * @package			sv_100
 * @copyright		2017 straightvisions GmbH
 * @link			https://straightvisions.com
 * @since			1.0
 * @license			See license.txt or https://straightvisions.com
 */

class sv_media_lib extends modules {
	public function init(){
		// Section Info
		$this->set_section_title( __( 'Media Library', 'sv_media_library' ) )
			->set_section_desc( __( 'Settings for Media Library', 'sv_media_library' ) )
			->set_section_type( 'settings' )
			->load_settings()
			->get_root()
			->add_section( $this );

		// Action Hooks
		// deactivate WPThumb plugin as it's not compatible with media lib feature
		// plugin-uri: https://wordpress.org/plugins/wp-thumb/
		remove_filter( 'image_downsize', 'wpthumb_post_image', 99 );

		return $this;
	}

	public function load_settings() :sv_media_lib{
		$this->get_setting( 'path' )
			->set_title( __( 'Media Lib Path', 'sv_media_library' ) )
			->set_description( __( 'Absolute path to media library base directory, replacing default ' , 'sv_media_library' ) . ABSPATH . '/wp-content/uploads/' )
			->load_type( 'text' );

		$this->get_setting( 'url' )
			->set_title( __( 'Media Lib URL', 'sv_media_library' ) )
			->set_description( __( 'Absolute URL to media library directory, replacing default ' , 'sv_media_library' ) . get_home_url() . '/wp-content/uploads/' )
			->load_type('text');

		$this->set_media_paths();

		return $this;
	}

	public function set_media_paths() :sv_media_lib{
		if ( $this->get_setting('url')->run_type()->get_data() && $this->get_setting('path')->run_type()->get_data() && file_exists($this->get_setting('path')->run_type()->get_data()) ) {
			add_filter( 'upload_dir', function( $param ) {
				$param['baseurl']		= $this->s['url']->run_type()->get_data();
				$param['basedir']		= $this->s['path']->run_type()->get_data();

				$param['path']			= $param['basedir'].$param['subdir'];
				$param['url']			= $param['baseurl'].$param['subdir'];

				return $param;
			} );

			add_filter( 'sv_100_get_url', array( $this, 'sv_100_get_url' ), 3, 5 );

			// set imagify backupdir within wp-content again to avoid bogus nor writable errors due to abspath check of imagify
			add_filter( 'imagify_backup_directory', function() { return trailingslashit( ABSPATH ) . 'wp-content/imagify-backups/'; } );
		}

		return $this;
	}

	public function sv_100_get_url( $url, $origin_url, $relative_path ) :string{
		$do_not_move					= array( 'js', 'css' );

		// we will copy theme media files to the new media storage domain to allow cookie less serving of static files
		$extension						= pathinfo( $relative_path )['extension'];

		if ( ! in_array( $extension, $do_not_move ) ) {
			// setup paths
			$module_relative_path       = str_replace( $this->get_root()->get_url(), '', $origin_url );
			$module_absolute_path       = $this->get_root()->get_path() . $module_relative_path;
			$media_lib_path             = trailingslashit( wp_upload_dir()['basedir'] ) . 'sv_media_library/';

			$old_path                   = $module_absolute_path . $relative_path;
			$new_path                   = $media_lib_path . $module_relative_path . $relative_path;

			$new_url                    = wp_upload_dir()['baseurl'] . '/sv_media_library/' . $module_relative_path . $relative_path;

			// copy files
			if ( ! file_exists( $new_path ) || md5_file( $old_path ) != md5_file( $new_path ) ) {
				// create directories
				mkdir( dirname( $new_path ), 0755, true );
				copy( $old_path, $new_path );
			}

			return $new_url;
		} else {
			return $url;
		}
	}
}